<?php if (!defined('ROOTPATH')) exit('No direct script access allowed'); ?>
<?php

require APPPATH . 'config/version.php';

// Make the version information available for error reports.
$version = $config['full'];

if (defined('STARTED_WITH_CLI'))
{
	require APPPATH . 'errors/cli.php';
}
else 
{
	$is_ajax = false;
	
	// We cannot use the request class here (request::is_ajax) because
	// an error may occur too early in the boot process to load this
	// class (e.g., when the requested controller does not exist).
	
	if (isset($_SERVER['HTTP_X_REQUESTED_WITH']))
	{
		$requested_with = $_SERVER['HTTP_X_REQUESTED_WITH'];
		if (str::to_lower($requested_with) == 'xmlhttprequest')
		{
			$is_ajax = true;
		}
	}
	
	$is_api = false;

	if (isset($_SERVER['CONTENT_TYPE']))
	{
		$content_type = str::to_lower($_SERVER['CONTENT_TYPE']);
		if ($content_type == 'application/json' || 
			$content_type == 'application/json;charset=utf-8' ||
			$content_type == 'application/json; charset=utf-8')
		{
			$is_api = true;
		}
	}

	$is_hosted = defined('DEPLOY_HOSTED') && DEPLOY_HOSTED;

	if ($is_ajax)
	{
		header('HTTP/1.1 200 OK');
		require APPPATH . 'errors/ajax.php';
	}
	else
	{
		switch ($status_code)
		{
			case 400:
				header('HTTP/1.1 400 Bad Request');
				break;

			case 401:
				header('HTTP/1.1 401 Unauthorized');
				break;

			case 403:
				header('HTTP/1.1 403 Forbidden');
				break;

			case 404:
				header('HTTP/1.1 404 File Not Found');
				break;

			case 500:
				if ($is_api || $is_hosted)
				{					
					header('HTTP/1.1 500 Internal Server Error');
				}
				else 
				{
					// In the UI case, we output the safer 200 OK as
					// some servers (e.g. IIS) may not display our HTML
					// page otherwise. Not required on TestRail Hosted.
					header('HTTP/1.1 200 OK');
				}
				break;

			case 503:
				if ($is_api || $is_hosted)
				{					
					header('HTTP/1.1 503 Service Unavailable');
				}
				else 
				{
					// In the UI case, we output the safer 200 OK as
					// some servers (e.g. IIS) may not display our HTML
					// page otherwise. Not required on TestRail Hosted.
					header('HTTP/1.1 200 OK');
				}
				break;				

			default:
				header('HTTP/1.1 200 OK');
				break;
		}
	
		if ($is_api)
		{
			require APPPATH . 'errors/api.php';
		}
		else 
		{
			// In case we deal with a generic 503 maintenance error (no
			// message) and are running on TestRail Hosted, we also
			// display a generic maintenance error page. In all other
			// cases, we display the full error page.
			if ($is_hosted && $status_code == 503 && !$message)
			{
				require APPPATH . 'errors/html_maint.php';
			}
			else 
			{
				require APPPATH . 'errors/html.php';			
			}
		}
	}
}
